from name import Name
from argument import Argument
from expression import Expression
from parameter_declarations import ParameterDeclarations
from xml_generation import xml_node


class Arguments:
    def __init__(self):
        self.__arguments: dict[Name, Expression] = dict()

    @property
    def list_of_arguments(self) -> list[Argument]:
        return list(map(lambda t: Argument(*t), self.__arguments.items()))

    def __contains__(self, name: Name) -> bool:
        return name in self.__arguments

    def __getitem__(self, name: Name) -> Argument:
        if not self.__contains__(name):
            raise KeyError('name "' + str(name) + '" not found in arguments')

        return Argument(name, self.__arguments[name])

    def add(self, argument: Argument) -> None:
        if self.__contains__(argument.name):
            raise ValueError

        self.__arguments[argument.name] = argument.expression

    def __len__(self) -> int:
        return len(self.__arguments)

    def __repr__(self) -> str:
        return "{" + ", ".join(map(str, self.list_of_arguments)) + "}"

    def to_xml(self) -> str:
        return xml_node("arguments",
                        '\n'.join(map(lambda x: x.to_xml(), self.list_of_arguments)))

    def names(self) -> list[Name]:
        return list(self.__arguments.keys())

    def expressions(self) -> list[Expression]:
        return list(self.__arguments.values())

    def match(self, parameters: ParameterDeclarations) -> bool:
        if len(self.__arguments) != len(parameters):
            return False

        for name in self.names():
            if name not in parameters:
                return False
        return True
