from dataclasses import dataclass

from expression import Expression
from name import Name

# TODO: should I differentiate between expressions and assignables/field accessibles?
# not on all expressions can field access occur: e.g. (x+y).z is invalid
# assignables can occur on both sides of the assignment,
# examples of assignables: a(i), a.b, a.y(i,j), and a(i).b.c(j).d(k)


@dataclass(frozen=True)
class DotAccess(Expression):
    expression: Expression
    name: Name

    def __repr__(self) -> str:
        return str(self.expression) + '.' + str(self.name)

    def is_closed(self) -> bool:
        return self.expression.is_closed()

    def contains_expression(self, expression) -> bool:
        return self == expression \
               or self.expression.contains_expression(expression)
