from copy import copy
from typing import List, Tuple

from assignments import Assignments
from expression import Expression
from partial_call_stack import PartialCallStack
from subprogram_declaration import SubprogramDeclaration


def _arguments_to_assignments(subprogram_call: SubprogramDeclaration.SubprogramCall) -> Assignments:
    parameter_declarations = subprogram_call.callee.parameter_declarations
    assignments = Assignments()
    for argument in subprogram_call.arguments.list_of_arguments:
        parameter_declaration = parameter_declarations.get_by_name(argument.name)
        parameter_reference = parameter_declaration.to_parameter_reference()
        assignments.add(parameter_reference, argument.expression)
    return assignments


def _recursively_expression_to_value_in_call_stack_pairs(
        maximally: bool,
        expression: Expression,
        subprogram_declaration: SubprogramDeclaration,
        partial_call_stack: PartialCallStack
) -> List[Tuple[Expression, PartialCallStack]]:
    if expression.is_closed():
        return [(expression, partial_call_stack)]

    if partial_call_stack.contains_subprogram_declaration(subprogram_declaration) or \
            len(subprogram_declaration.to_calls) == 0:
        if maximally:
            return [(expression, partial_call_stack)]
        return []

    return_value = list()
    for call in subprogram_declaration.to_calls:
        assignments = _arguments_to_assignments(call)
        new_expression = assignments.substitute_in_expression(expression)
        new_call_stack = copy(partial_call_stack)
        new_call_stack.push(call)
        values = _recursively_expression_to_value_in_call_stack_pairs(maximally,
                                                                      new_expression,
                                                                      call.caller,
                                                                      new_call_stack)
        return_value.extend(values)
    return return_value


def expression_to_value_in_call_stack_pairs(expression: Expression,
                                            subprogram_declaration: SubprogramDeclaration) \
        -> List[Tuple[Expression, PartialCallStack]]:
    return _recursively_expression_to_value_in_call_stack_pairs(False,
                                                                expression,
                                                                subprogram_declaration,
                                                                PartialCallStack())


def expression_to_values(expression: Expression,
                         subprogram_declaration: SubprogramDeclaration) -> List[Expression]:
    return [t[0]
            for t in expression_to_value_in_call_stack_pairs(expression,
                                                             subprogram_declaration)]


def maximally_resolve_expression_in_call_stack_pairs(
        expression: Expression,
        subprogram_declaration: SubprogramDeclaration
) -> List[Tuple[Expression, PartialCallStack]]:
    return _recursively_expression_to_value_in_call_stack_pairs(True,
                                                                expression,
                                                                subprogram_declaration,
                                                                PartialCallStack())


def maximally_resolve_expression(expression: Expression,
                                 subprogram_declaration: SubprogramDeclaration) -> List[Expression]:
    return [t[0]
            for t in maximally_resolve_expression_in_call_stack_pairs(expression,
                                                                      subprogram_declaration)]
