from dot_access import DotAccess
from expression_to_values import maximally_resolve_expression_in_call_stack_pairs
from extract_symbolic_result import ExtractSymbolicResult
from index_access import IndexAccess
from inspection_app import InspectionApp
from inspection_apps import InspectionApps
from inspection_class import InspectionClass
from inspection_classes import InspectionClasses
from inspection_proc import InspectionProc
from literal import Literal
from name import Name
from setter import Setter
from subprogram_declaration import SubprogramDeclaration
from tuple_expression import TupleExpression
from variable import Variable


def has_parameter_with_type(subprogram_declaration: SubprogramDeclaration,
                            parameter_type: Name) \
        -> bool:
    return parameter_type in subprogram_declaration.parameter_declarations.type_names()


def extract_inspection_classes(extract_symbolic_result: ExtractSymbolicResult) -> InspectionClasses:
    inspection_classes = InspectionClasses()
    for file in extract_symbolic_result.files.files:
        for subprogram_declaration in file.subprogram_declarations:
            for init_inspection_object_common in \
                    subprogram_declaration.init_inspection_object_commons:
                tuple_expression = TupleExpression()
                tuple_expression.append(init_inspection_object_common.class_expression)
                tuple_expression.append(init_inspection_object_common.name)
                resolved_expression_in_call_stack_pairs = \
                    maximally_resolve_expression_in_call_stack_pairs(tuple_expression,
                                                                     subprogram_declaration)
                for (resolved_expression, partial_call_stack) \
                        in resolved_expression_in_call_stack_pairs:
                    assert isinstance(resolved_expression, TupleExpression), \
                        "Expected TupleExpression, yet is " + str(type(resolved_expression))
                    [class_value, name_value] = resolved_expression.sequence
                    if inspection_classes.contains(class_value):
                        inspection_classes.get_inspection_class(class_value).add_name(name_value)
                    else:
                        inspection_class = InspectionClass(class_value)
                        inspection_class.add_name(name_value)
                        inspection_classes.add_inspection_class(inspection_class)
    return inspection_classes


def extract_inspection_apps(extract_symbolic_result: ExtractSymbolicResult,
                            inspection_classes: InspectionClasses) -> InspectionApps:
    inspection_apps = InspectionApps()
    for file in extract_symbolic_result.files.files:
        for subprogram_declaration in file.subprogram_declarations:
            if subprogram_declaration.name == Name("Init_Vision_Applic"):
                tuple_expression = TupleExpression()
                tuple_expression.append(
                    subprogram_declaration.parameter_declarations
                        .get_by_name(Name("V"))
                        .to_parameter_reference()
                )
                tuple_expression.append(
                    subprogram_declaration.parameter_declarations
                        .get_by_name(Name("Name"))
                        .to_parameter_reference()
                )
                resolved_expression_in_call_stack_pairs = \
                    maximally_resolve_expression_in_call_stack_pairs(tuple_expression,
                                                                     subprogram_declaration)
                for (resolved_expression, partial_call_stack) \
                        in resolved_expression_in_call_stack_pairs:
                    assert isinstance(resolved_expression, TupleExpression), \
                        "Expected TupleExpression, yet is " + str(type(resolved_expression))
                    [v_value, name_value] = resolved_expression.sequence
                    if inspection_apps.contains(v_value):
                        inspection_apps.get_inspection_app(v_value).add_name(name_value)
                    else:
                        inspection_app = InspectionApp(v_value)
                        inspection_app.add_name(name_value)
                        inspection_apps.add_inspection_app(inspection_app)

                        for inspection_class in inspection_classes.inspection_classes:
                            if v_value.contains_expression(inspection_class.class_expression):
                                inspection_class.add_inspection_app(inspection_app)
                    # if partial_call_stack.starts_with(inspection_class.partial_call_stack):
                    #     print("In call stack")
                    # else:
                    #     print("Not in call stack")
    return inspection_apps


def extract_inspection_procs(extract_symbolic_result: ExtractSymbolicResult,
                             inspection_apps: InspectionApps) -> None:
    for file in extract_symbolic_result.files.files:
        if file.file_path.canonical_path.endswith("visapp.ads"):
            for subprogram_declaration in file.subprogram_declarations:
                if subprogram_declaration.name == Name("set_process"):
                    tuple_expression = TupleExpression()
                    tuple_expression.append(
                        subprogram_declaration.parameter_declarations
                            .get_by_name(Name("V"))
                            .to_parameter_reference()
                    )
                    tuple_expression.append(
                        subprogram_declaration.parameter_declarations
                            .get_by_name(Name("P"))
                            .to_parameter_reference()
                    )
                    tuple_expression.append(
                        subprogram_declaration.parameter_declarations
                            .get_by_name(Name("P_Index"))
                            .to_parameter_reference()
                    )
                    tuple_expression.append(
                        subprogram_declaration.parameter_declarations
                            .get_by_name(Name("Name"))
                            .to_parameter_reference()
                    )
                    resolved_expression_in_call_stack_pairs = \
                        maximally_resolve_expression_in_call_stack_pairs(tuple_expression,
                                                                         subprogram_declaration)
                    for (resolved_expression, partial_call_stack) \
                            in resolved_expression_in_call_stack_pairs:
                        assert isinstance(resolved_expression, TupleExpression), \
                            "Expected TupleExpression, yet is " + str(type(resolved_expression))
                        [v_value, p_value, p_index_value, name_value] = resolved_expression.sequence
                        if p_value == Literal("Template"):
                            kind_value = Literal("Match")
                        elif p_value == Literal("Edge"):
                            kind_value = Literal("Edge")
                        elif p_value == Literal("Pixel"):
                            kind_value = Literal("Pixel")
                        elif p_value == Literal("Blob"):
                            kind_value = Literal("Blob")
                        elif p_value in [Literal("Rectangle"), Literal("Circle"), Literal("Line")]:
                            kind_value = Literal("Ruler")
                        else:
                            kind_value = Literal("Unknown")

                        if inspection_apps.contains(v_value):
                            inspection_app = inspection_apps.get_inspection_app(v_value)
                            inspection_procs = inspection_app.procs
                            if inspection_procs.contains(kind_value.literal, p_index_value):
                                inspection_proc = \
                                    inspection_procs.get_inspection_proc(kind_value.literal,
                                                                         p_index_value)
                            else:
                                inspection_proc = InspectionProc(kind_value.literal, p_index_value)
                                inspection_procs.add_inspection_proc(inspection_proc)
                            inspection_proc.add_name(name_value)
                            inspection_proc.add_sub_kind(p_value)


def extract_param_assignments(extract_symbolic_result: ExtractSymbolicResult,
                              inspection_apps: InspectionApps) -> None:
    for file in extract_symbolic_result.files.files:
        for subprogram_declaration in file.subprogram_declarations:
            for param_assignment in subprogram_declaration.param_assignments:
                tuple_expression = TupleExpression()
                tuple_expression.append(param_assignment.app_expression)
                tuple_expression.append(param_assignment.attribute_expression)
                tuple_expression.append(param_assignment.value_expression)
                resolved_expression_in_call_stack_pairs = \
                    maximally_resolve_expression_in_call_stack_pairs(tuple_expression,
                                                                     subprogram_declaration)
                for (resolved_expression,
                     partial_call_stack) in resolved_expression_in_call_stack_pairs:
                    assert isinstance(resolved_expression, TupleExpression), \
                        "Expected TupleExpression, yet is " + str(type(resolved_expression))
                    [app_value, attribute_value, value_value] = resolved_expression.sequence
                    if inspection_apps.contains(app_value):
                        inspection_app = inspection_apps.get_inspection_app(app_value)
                        if type(attribute_value) == DotAccess:
                            setter = Setter(attribute_value.name,
                                            value_value,
                                            param_assignment.file_path,
                                            param_assignment.file_range)
                            if type(attribute_value.expression) == IndexAccess and \
                                    type(attribute_value.expression.expression) == IndexAccess:
                                # [proc].par(Ruler)(1).specific
                                inspection_procs = inspection_app.procs
                                # Why not iterate over all indices? Why only 0?
                                kind_value = attribute_value.expression.expression.indices[0]
                                p_index_value = attribute_value.expression.indices[0]
                                if inspection_procs.contains(kind_value, p_index_value):
                                    inspection_proc = inspection_procs.get_inspection_proc(
                                        kind_value, p_index_value)
                                    inspection_proc.setters.add(setter)
                            elif type(attribute_value.expression) == Variable:
                                # [proc].aoi_modify
                                inspection_app.setters.add(setter)


def extract_vis_app_calls(extract_symbolic_result: ExtractSymbolicResult,
                          inspection_apps: InspectionApps) -> None:
    for file in extract_symbolic_result.files.files:
        if file.file_path.canonical_path.endswith("visapp.ads"):
            for subprogram_declaration in file.subprogram_declarations:
                if has_parameter_with_type(subprogram_declaration, Name("Vis_App_Cwa")):
                    tuple_expression = TupleExpression()
                    tuple_expression.append(
                        subprogram_declaration.parameter_declarations
                            .get_by_name(Name("V"))
                            .to_parameter_reference()
                    )
                    resolved_expression_in_call_stack_pairs = \
                        maximally_resolve_expression_in_call_stack_pairs(tuple_expression,
                                                                         subprogram_declaration)
                    for (resolved_expression, partial_call_stack) \
                            in resolved_expression_in_call_stack_pairs:
                        assert isinstance(resolved_expression, TupleExpression), \
                            "Expected TupleExpression, yet is " + str(type(resolved_expression))
                        [v_value] = resolved_expression.sequence
                        if inspection_apps.contains(v_value):
                            inspection_app = inspection_apps.get_inspection_app(v_value)
                            inspection_app.add_call(partial_call_stack.stack[0])
                        else:
                            print("Inspection Apps don't contain " + str(v_value))


def main():
    extract_symbolic_result = ExtractSymbolicResult()
    inspection_classes = extract_inspection_classes(extract_symbolic_result)
    inspection_apps = extract_inspection_apps(extract_symbolic_result, inspection_classes)
    extract_inspection_procs(extract_symbolic_result, inspection_apps)
    extract_param_assignments(extract_symbolic_result, inspection_apps)
    extract_vis_app_calls(extract_symbolic_result, inspection_apps)
    print("nrof apps " + str(len(inspection_apps.inspection_apps)))
    print(inspection_apps.to_xml())
    print("====================================================")
    print(inspection_classes.to_xml())


if __name__ == "__main__":
    main()
