from file_path import FilePath
from file_range import FileRange
from subprogram_declaration import SubprogramDeclaration


class File:
    def __init__(self, file_path: FilePath):
        self.__file_path: FilePath = file_path
        self.__subprogram_declarations: dict[FileRange, SubprogramDeclaration] = {}

    @property
    def file_path(self) -> FilePath:
        return self.__file_path

    @property
    def subprogram_declarations(self) -> list[SubprogramDeclaration]:
        return list(self.__subprogram_declarations.values())

    def __contains__(self, file_range: FileRange) -> bool:
        return file_range in self.__subprogram_declarations

    def __getitem__(self, file_range: FileRange) -> SubprogramDeclaration:
        return self.__subprogram_declarations[file_range]

    def add_subprogram_declaration(self, subprogram_declaration: SubprogramDeclaration) -> None:
        if self.__file_path != subprogram_declaration.file_path:
            raise ValueError('Cannot add subprogram declaration to another file.')

        if self.__contains__(subprogram_declaration.file_range):
            raise ValueError('Subprogram declaration with identical file range already added.')

        self.__subprogram_declarations[subprogram_declaration.file_range] = subprogram_declaration
