import os.path


class FilePath:
    def __init__(self, file_path: str):
        def canonical_path() -> str:
            return os.path.normpath(os.path.normcase(file_path))

        # I would prefer to use os.path.samefile for equality
        # however the necessary corresponding hash function is not provided
        self.__canonical_path = canonical_path()

    @property
    def canonical_path(self) -> str:
        return self.__canonical_path

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, FilePath):
            return False

        return self.canonical_path == other.canonical_path

    def __hash__(self):
        return hash(self.__canonical_path)

    def __repr__(self) -> str:
        return self.__canonical_path
