from dataclasses import dataclass
from typing import AnyStr, Match, Tuple
import re

separator: str = ":"


@dataclass(frozen=True, order=True)
class FilePosition:
    line: int
    column: int

    def __post_init__(self):
        def valid_file_position(line: int, column: int) -> bool:
            return line > 0 and column > 0

        if not valid_file_position(self.line, self.column):
            raise ValueError("FilePosition constructor: invalid values " +
                             str(self.line) + separator + str(self.column))

    def __str__(self) -> str:
        return str(self.line) + separator + str(self.column)


pattern = re.compile("(\\d+)" + separator + "(\\d+)")


def __extract_file_position(match: Match[AnyStr]) -> Tuple[int, int]:
    return int(match.group(1)), int(match.group(2))


def parse(value: str) -> FilePosition:
    match = re.fullmatch(pattern, value)
    if match is None:
        raise ValueError("parse: invalid string format " + value)

    line, column = __extract_file_position(match)
    return FilePosition(line, column)
