from dataclasses import dataclass
from file_position import FilePosition
import file_position


separator: str = "-"


@dataclass(frozen=True)
class FileRange:
    start: FilePosition
    end: FilePosition

    def __post_init__(self):
        def valid_file_range(start: FilePosition, end: FilePosition) -> bool:
            return start < end

        if not valid_file_range(self.start, self.end):
            raise ValueError("FileRange constructor: " + str(self.start) + ">=" + str(self.end))

    def __str__(self) -> str:
        return str(self.start) + separator + str(self.end)

    def contains(self, other: object) -> bool:
        if not isinstance(other, FileRange):
            return False

        return not(other.start < self.start or self.end < other.end)


def parse(value: str) -> FileRange:
    elements = value.split(separator)
    if len(elements) != 2:
        raise ValueError("parse: invalid string format " + value)

    return FileRange(file_position.parse(elements[0]),
                     file_position.parse(elements[1]))
