from typing import Dict, List

from file import File
from file_path import FilePath
from file_range import FileRange
from subprogram_declaration import SubprogramDeclaration


class Files:
    def __init__(self):
        self.__files: Dict[FilePath, File] = {}

    @property
    def files(self) -> List[File]:
        return list(self.__files.values())

    def __contains__(self, file_path: FilePath) -> bool:
        return file_path in self.__files

    def __getitem__(self, file_path: FilePath) -> File:
        return self.__files[file_path]

    def add(self, file: File) -> None:
        if self.__contains__(file.file_path):
            raise ValueError

        self.__files[file.file_path] = file

    def __len__(self):
        return len(self.__files)

    def subprogram_declaration(self, file_path: FilePath, file_range: FileRange) \
            -> SubprogramDeclaration:
        return self.__files[file_path][file_range]

    def nr_of_subprogram_declarations(self) -> int:
        return sum(map(lambda file: len(file.subprogram_declarations),
                       self.__files.values()))

    def nr_of_subprogram_calls(self) -> int:
        return sum(map(lambda file: sum(map(lambda sp: len(sp.to_calls),
                                            file.subprogram_declarations)),
                       self.__files.values()))

    def nr_of_init_inspection_object_commons(self) -> int:
        return sum(map(lambda file: sum(map(lambda sp: len(sp.init_inspection_object_commons),
                                            file.subprogram_declarations)),
                       self.__files.values()))
