from typing import List
from expression import Expression


class IndexAccess(Expression):
    def __init__(self, expression: Expression, indices: List[Expression]) -> None:
        if len(indices) == 0:
            raise ValueError("At least one index expected in IndexAccess")
        self.__expression: Expression = expression
        self.__indices: List[Expression] = indices

    @property
    def expression(self) -> Expression:
        return self.__expression

    @property
    def indices(self) -> List[Expression]:
        return self.__indices

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, IndexAccess):
            return False

        if self.expression != other.expression:
            return False

        if len(self.indices) != len(other.indices):
            return False

        for (self_index, other_index) in zip(self.indices, other.indices):
            if self_index != other_index:
                return False

        return True

    def __hash__(self) -> int:
        return_value = hash(self.__expression)
        for index in self.__indices:
            return_value = return_value ^ hash(index)
        return return_value

    def __repr__(self) -> str:
        return str(self.__expression) + '(' + ','.join(map(str, self.__indices)) + ")"

    def is_closed(self) -> bool:
        return self.__expression.is_closed() and all(map(lambda x: x.is_closed(), self.__indices))

    def contains_expression(self, expression) -> bool:
        return self == expression \
               or self.expression.contains_expression(expression) \
               or any(map(lambda x: x.contains_expression(expression), self.__indices))
