from collections import Counter
from name import Name
from expression import Expression
from subprogram_declaration import SubprogramDeclaration
from inspection_procs import InspectionProcs
from setter import Setter
from xml_generation import xml_node


class InspectionApp:
    def __init__(self,
                 v_expression: Expression
                 ):
        self.__v_expression: Expression = v_expression
        self.__names: set[Name] = set()
        self.__calls: Counter[SubprogramDeclaration.SubprogramCall] = Counter()
        self.__setters: set[Setter] = set()
        self.__inspection_procs: InspectionProcs = InspectionProcs()

    @property
    def v_expression(self) -> Expression:
        return self.__v_expression

    @property
    def names(self) -> set[Name]:
        return self.__names

    @property
    def setters(self) -> set[Setter]:
        return self.__setters

    @property
    def procs(self) -> InspectionProcs:
        return self.__inspection_procs

    def add_name(self, name: Name) -> None:
        self.__names.add(name)

    def add_call(self, call: SubprogramDeclaration.SubprogramCall) -> None:
        self.__calls[call] += 1

    def to_xml(self) -> str:
        def item_to_xml(element, count: int) -> str:
            if count == 1:
                return element.to_xml()

            return xml_node("occurrence", element.to_xml(), {"count": str(count)})

        attributes: dict[str, str] = {"v": str(self.v_expression)}
        content: list[str] = []
        if len(self.__names) > 0:
            if len(self.__names) == 1:
                attributes["name"] = str(list(self.__names)[0])
            else:
                content.append(
                    xml_node("names",
                             "\n".join(map(lambda x: xml_node("name", str(x)), self.__names))))

        if len(self.__calls) > 0:
            content.append(
                xml_node("calls",
                         "\n".join(map(lambda item: item_to_xml(*item),
                                       self.__calls.items()))))

        if len(self.__setters) > 0:
            content.append(
                xml_node("setters", "\n".join(map(lambda x: x.to_xml(), self.__setters))))

        if len(self.__inspection_procs) > 0:
            content.append(self.__inspection_procs.to_xml())

        return xml_node("inspectApp",
                        xml='\n'.join(content),
                        attributes=attributes)
