from typing import Dict, List
from expression import Expression
from inspection_app import InspectionApp
from xml_generation import xml_node

class InspectionApps:
    def __init__(self):
        self.__inspection_apps: Dict[Expression, InspectionApp] = dict()

    @property
    def inspection_apps(self) -> List[InspectionApp]:
        return list(self.__inspection_apps.values())

    def contains(self, expression: Expression) -> bool:
        return expression in self.__inspection_apps

    def get_inspection_app(self, expression: Expression) -> InspectionApp:
        return self.__inspection_apps[expression]

    def add_inspection_app(self, inspection_app: InspectionApp) -> None:
        if self.contains(inspection_app.v_expression):
            raise ValueError("Expression already in InspectionApps")
        self.__inspection_apps[inspection_app.v_expression] = inspection_app

    def __len__(self):
        return len(self.__inspection_apps)

    def to_xml(self) -> str:
        return xml_node('inspectApps',
                        '\n'.join(map(lambda x: x.to_xml(), self.__inspection_apps.values())))
