from typing import Set

from expression import Expression
from inspection_app import InspectionApp
from inspection_apps import InspectionApps
from name import Name
from xml_generation import xml_node


class InspectionClass:
    def __init__(self,
                 class_expression: Expression
                 ):
        self.__class_expression: Expression = class_expression
        self.__names: Set[Name] = set()
        self.__inspection_apps: InspectionApps = InspectionApps()

    @property
    def class_expression(self) -> Expression:
        return self.__class_expression

    @property
    def names(self) -> Set[Name]:
        return self.__names

    @property
    def inspection_apps(self) -> InspectionApps:
        return self.__inspection_apps

    def add_name(self, name: Name) -> None:
        self.__names.add(name)

    def add_inspection_app(self, inspection_app: InspectionApp) -> None:
        if not inspection_app.v_expression.contains_expression(self.__class_expression):
            raise ValueError("Class expression " +
                             "'" + str(self.__class_expression) + "'" +
                             " not contained in app expression " +
                             "'" + str(inspection_app.v_expression) + "'")
        if not self.__inspection_apps.contains(inspection_app.v_expression):
            self.__inspection_apps.add_inspection_app(inspection_app)
        else:
            print("Already added " + str(inspection_app.v_expression) +
                  " to " + str(self.__class_expression))

    def to_xml(self) -> str:
        attributes: dict[str, str] = {"class": str(self.class_expression)}
        content: list[str] = []

        if len(self.__names) > 0:
            if len(self.__names) == 1:
                attributes["name"] = str(list(self.__names)[0])
            else:
                content.append(
                    xml_node("names",
                             "".join(map(lambda x: xml_node('name', str(x)), self.__names))))

        if len(self.__inspection_apps) > 0:
            content.append(self.__inspection_apps.to_xml())

        return xml_node(tag='inspectionClass',
                        xml='\n'.join(content),
                        attributes=attributes
                        )
