from typing import Dict, List
from expression import Expression
from inspection_class import InspectionClass
from xml_generation import xml_node


class InspectionClasses:
    def __init__(self):
        self.__inspection_classes: Dict[Expression, InspectionClass] = dict()

    @property
    def inspection_classes(self) -> List[InspectionClass]:
        return list(self.__inspection_classes.values())

    def contains(self, expression: Expression) -> bool:
        return expression in self.__inspection_classes

    def get_inspection_class(self, expression: Expression) -> InspectionClass:
        return self.__inspection_classes[expression]

    def add_inspection_class(self, inspection_class: InspectionClass) -> None:
        if self.contains(inspection_class.class_expression):
            raise ValueError("Expression already in InspectionClasses")
        self.__inspection_classes[inspection_class.class_expression] = inspection_class

    def __len__(self):
        return len(self.__inspection_classes)

    def to_xml(self) -> str:
        return xml_node('inspectClasses',
                        '\n'.join(map(lambda x: x.to_xml(), self.__inspection_classes.values()))
                        )
