from name import Name
from setter import Setter
from xml_generation import xml_node


class InspectionProc:
    def __init__(self,
                 kind: str,
                 index: int
                 ):
        self.__kind: str = kind
        self.__index: int = index
        self.__names: set[Name] = set()
        self.__sub_kinds: set[Name] = set()
        self.__setters: set[Setter] = set()

    @property
    def kind(self) -> str:
        return self.__kind

    @property
    def index(self) -> int:
        return self.__index

    @property
    def names(self) -> set[Name]:
        return self.__names

    @property
    def sub_kinds(self) -> set[Name]:
        return self.__sub_kinds

    @property
    def setters(self) -> set[Setter]:
        return self.__setters

    def add_name(self, name: Name) -> None:
        self.__names.add(name)
        
    def add_sub_kind(self, sub_kind: Name) -> None:
        self.__sub_kinds.add(sub_kind)

    def to_xml(self) -> str:
        attributes: dict[str, str] = {"kind": str(self.kind), "index": str(self.index)}
        content: list[str] = []
        name_tag = 'name'
        if len(self.__names) > 0:
            if len(self.__names) == 1:
                attributes[name_tag] = str(list(self.__names)[0])
            else:
                content.append(
                    xml_node("names",
                             "\n".join(map(lambda x: xml_node(name_tag, str(x)), self.__names))))

        sub_kind_tag = 'subkind'
        if len(self.__sub_kinds) > 0:
            if len(self.__sub_kinds) == 1:
                attributes[sub_kind_tag] = str(list(self.__sub_kinds)[0])
            else:
                content.append(
                    xml_node("subkinds",
                             "\n".join(map(lambda x: xml_node(sub_kind_tag, str(x)),
                                           self.__sub_kinds))))

        if len(self.__setters) > 0:
            content.append(xml_node("setters",
                                    "\n".join(map(lambda x: x.to_xml(), self.__setters))))

        return xml_node(tag="inspectProc",
                        xml='\n'.join(content),
                        attributes=attributes
                        )

