from inspection_proc import InspectionProc
from xml_generation import xml_node


class InspectionProcs:
    def __init__(self):
        self.__inspection_procs: dict[tuple[str, int], InspectionProc] = dict()

    @property
    def inspection_procs(self) -> list[InspectionProc]:
        return list(self.__inspection_procs.values())

    def contains(self, kind: str, index: int) -> bool:
        return (kind, index) in self.__inspection_procs

    def get_inspection_proc(self, kind: str, index: int) -> InspectionProc:
        return self.__inspection_procs[(kind, index)]

    def add_inspection_proc(self, inspection_proc: InspectionProc) -> None:
        if self.contains(inspection_proc.kind, inspection_proc.index):
            raise ValueError("Kind/Index already in InspectionProcs")
        self.__inspection_procs[(inspection_proc.kind, inspection_proc.index)] = inspection_proc

    def __len__(self):
        return len(self.__inspection_procs)

    def to_xml(self) -> str:
        return xml_node('inspectProcs',
                        '\n'.join(map(lambda x: x.to_xml(), self.__inspection_procs.values())))
