from typing import List
from subprogram_declaration import SubprogramDeclaration


class PartialCallStack:
    def __init__(self):
        self.__stack: List[SubprogramDeclaration.SubprogramCall] = list()

    def __copy__(self):
        return_value = PartialCallStack()
        return_value.__stack = self.__stack.copy()
        return return_value

    @property
    def stack(self) -> List[SubprogramDeclaration.SubprogramCall]:
        return list(self.__stack)

    def __repr__(self) -> str:
        return "[ " + "\n  ".join(map(str, self.__stack)) + "\n]"

    def push(self,
             call: SubprogramDeclaration.SubprogramCall) -> None:
        self.__stack.append(call)

    def contains_subprogram_declaration(self, callee: SubprogramDeclaration) -> bool:
        for call in self.__stack:
            if callee == call.callee:
                return True
        return False

    def starts_with(self, other: object) -> bool:
        if not isinstance(other, PartialCallStack):
            raise ValueError("Unexpected type for another " + str(type(other)))

        if len(other.stack) > len(self.__stack):
            return False

        for (si, osi) in zip(self.__stack, other.stack):
            if si != osi:
                return False
        return True
