from abc import ABC
from dataclasses import dataclass
from expression import Expression
from file_path import FilePath
from file_range import FileRange
from name import Name


@dataclass(frozen=True)
class Reference(Expression, ABC):
    """
    Since nested functions in Ada can refer to a hidden homonym in an enclosing scope
    by using the name of the enclosing scope as a prefix,
    we need similar functionality.
    We realize the distinction of references by adding file path and file range to the name.
    """
    name: Name
    file_path: FilePath
    file_range: FileRange

    def __str__(self) -> str:
        return '[' + str(self.name) + ']'

    def is_closed(self) -> bool:
        return False

    def contains_expression(self, expression) -> bool:
        return self == expression
