from abc import ABC
from dataclasses import dataclass
from expression import Expression
from literal import Literal


@dataclass(frozen=True)
class Sequence(Expression, ABC):
    sequence: tuple[Expression, ...]

    def __len__(self) -> int:
        return len(self.sequence)

    def is_closed(self) -> bool:
        return all(map(lambda x: x.is_closed(), self.sequence))

    def contains_expression(self, expression: Expression) -> bool:
        return self == expression or \
               any(map(lambda x: x.contains_expression(expression), self.sequence))

    def is_literal(self) -> bool:
        return all(map(lambda x: isinstance(x, Literal), self.sequence))

    def to_literal(self) -> Literal:
        """if is_literal(self):
                return a literal of the concatenated literal values
           else:
                raise an ValueError
        """
        return_string = ""
        for elem in self.sequence:
            if isinstance(elem, Literal):
                return_string += elem.literal
            else:
                raise ValueError
        return Literal(return_string)
