from typing import Optional
from xml.etree.ElementTree import Element


def get_single_child_node(node: Element) -> Element:
    if len(node) != 1:
        raise ValueError("not a single node, but " + str(len(node)))
    return node[0]


def get_single_child_node_with_name(node: Element, child_name: str) -> Element:
    child_nodes = node.findall(child_name)
    if len(child_nodes) != 1:
        raise ValueError("not a single node with the given name, but " + str(len(child_nodes)))
    return child_nodes[0]


def get_optional_child_node_with_name(node: Element, child_name: str) -> Optional[Element]:
    child_nodes = node.findall(child_name)
    length = len(child_nodes)
    if length > 1:
        raise ValueError("not an optional node with the given name, but " + str(len(child_nodes)))

    if length == 0:
        return None

    return child_nodes[0]


def get_text_of_node(node: Element) -> str:
    if node.text is None:
        return ""

    return node.text
