from typing import Optional
from re import sub


def __xml_tag_with_attributes(tag: str,
                              attributes: dict[str, str]) -> str:
    elements = [tag]
    elements.extend(map(lambda item: '{0}="{1}"'.format(*item), attributes.items()))
    return " ".join(elements)


def _xml_leaf(tag: str,
              attributes: dict[str, str]
              ) -> str:
    return '<{0}/>'.format(__xml_tag_with_attributes(tag, attributes))


def _xml_node(tag: str,
              xml: str,
              attributes: dict[str, str]
              ) -> str:
    def has_multi_lines() -> bool:
        return '\n' in xml

    def indent(lines: str) -> str:
        indentation = '   '
        return indentation + sub('\n', '\n' + indentation, lines)

    def multi_line_node() -> str:
        return '<{0}>\n{1}\n</{2}>'.format(
            __xml_tag_with_attributes(tag, attributes),
            indent(xml),
            tag
        )

    def single_line_node() -> str:
        return '<{0}>{1}</{2}>'.format(
            __xml_tag_with_attributes(tag, attributes),
            xml,
            tag
        )

    if has_multi_lines():
        return multi_line_node()
    return single_line_node()


# TODO: check tag and attributes name adhere to xml standard
# TODO: encode attribute values (e.g. '<' -> '&lt;')
def xml_node(tag: str,
             xml: str = "",
             attributes: Optional[dict[str, str]] = None
             ) -> str:
    if attributes is None:
        attributes = dict()

    return _xml_leaf(tag, attributes) if xml == "" else _xml_node(tag, xml, attributes)
